@@ -1,7 +1,7 @@
 /*! ========================================================================
 ** Extended Template and Library
 ** B-Spline Class Implementation
-** $Id: _bspline.h 334 2007-03-16 00:37:48Z dooglus $
+** $Id$
 **
 ** Copyright (c) 2002 Robert B. Quattlebaum Jr.
 **
@@ -116,7 +116,8 @@ public:
 			t=0;
 		if(t>=_knots.back())
 			t=_knots.back()-0.0001;
-		for(k=0;_knots[k]>t || _knots[k+1]<=t;k++);
+		for(k=0;_knots[k]>t || _knots[k+1]<=t;k++)
+		  ;
 
 		return k;
 	}
