@@ -1,11 +1,12 @@
 /* === E T L =============================================================== */
 /*!	\file _handle.h
-**	$Id: _handle.h 659 2007-09-12 21:01:33Z dooglus $
+**	$Id$
 **	\brief Template Object Handle Implementation
 **	\internal
 **
 **	\legal
 **	Copyright (c) 2002 Robert B. Quattlebaum Jr.
+**	Copyright (c) 2007, 2008 Chris Moore
 **
 **	This package is free software; you can redistribute it and/or
 **	modify it under the terms of the GNU General Public License as
@@ -66,6 +67,9 @@ class shared_object
 {
 private:
 	mutable int refcount;
+#ifdef ETL_LOCK_REFCOUNTS
+	mutable etl::mutex mtx;
+#endif
 
 protected:
 	shared_object():refcount(0) { }
@@ -78,28 +82,43 @@ protected:
 
 public:
 	void ref()const
-		{ assert(refcount>=0); refcount++; }
+	{
+#ifdef ETL_LOCK_REFCOUNTS
+		etl::mutex::lock lock(mtx);
+#endif
+		assert(refcount>=0);
+		refcount++;
+	}
 
 	//! Returns \c false if object needs to be deleted
 	bool unref()const
 	{
-		assert(refcount>0);
-
-		refcount--;
-
-		if(refcount==0) {
-#ifdef ETL_SELF_DELETING_SHARED_OBJECT
-			refcount=-666;
-			delete this;
+		bool ret = true;
+		{
+#ifdef ETL_LOCK_REFCOUNTS
+			etl::mutex::lock lock(mtx);
 #endif
-			return false;
+			assert(refcount>0);
+
+			refcount--;
+
+			if(refcount==0) {
+				ret = false;
+#ifdef ETL_SELF_DELETING_SHARED_OBJECT
+				refcount=-666;
+#endif
+			}
 		}
 
-		return true;
+#ifdef ETL_SELF_DELETING_SHARED_OBJECT
+		if (!ret)
+			delete this;
+#endif
+		return ret;
 	}
 
-	int count()const
-		{ return refcount; }
+	int count()const { return refcount; }
+
 }; // END of class shared_object
 
 // ========================================================================
@@ -140,6 +159,9 @@ public:
 	typedef int size_type;
 
 protected:
+#ifdef _DEBUG
+public:
+#endif
 	value_type *obj;		//!< Pointer to object
 
 public:
@@ -202,7 +224,7 @@ public:
 	swap(handle<value_type> &x)
 	{
 		pointer ptr=x.obj;
-		x.obj=x.get();
+		x.obj=obj;
 		obj=ptr;
 		return *this;
 	}
@@ -264,41 +286,29 @@ public:
 	operator handle<const value_type>()const
 	{ return handle<const value_type>(static_cast<const_pointer>(obj)); }
 
-
 	//! <tt> static_cast\<\> </tt> wrapper
-	template <class U> static
-	handle<T> cast_static(const handle<U> &x)
-	{ return handle<T>(static_cast<T*>(x.get())); }
-
+	template <class U> static handle<T> cast_static		(const handle<U> &x) { return handle<T>(static_cast		<T*>(x.get())); }
 	//! <tt> dynamic_cast\<\> </tt> wrapper
-	template <class U> static
-	handle<T> cast_dynamic(const handle<U> &x)
-	{ return handle<T>(dynamic_cast<T*>(x.get())); }
-
+	template <class U> static handle<T> cast_dynamic	(const handle<U> &x) { return handle<T>(dynamic_cast	<T*>(x.get())); }
 	//! <tt> const_cast\<\> </tt> wrapper
-	template <class U> static
-	handle<T> cast_const(const handle<U> &x)
-	{ return handle<T>(const_cast<T*>(x.get())); }
-
+	template <class U> static handle<T> cast_const		(const handle<U> &x) { return handle<T>(const_cast		<T*>(x.get())); }
 	//! <tt> reinterpret_cast\<\> </tt> wrapper
-	template <class U> static
-	handle<T> cast_reinterpret(const handle<U> &x)
-	{ return handle<T>(reinterpret_cast<T*>(x.get())); }
+	template <class U> static handle<T> cast_reinterpret(const handle<U> &x) { return handle<T>(reinterpret_cast<T*>(x.get())); }
 
-	template<class U> static handle<T> cast_static(const loose_handle<U> &x);
-	template<class U> static handle<T> cast_dynamic(const loose_handle<U> &x);
-	template<class U> static handle<T> cast_const(const loose_handle<U> &x);
-	template<class U> static handle<T> cast_reinterpret(const loose_handle<U> &x);
+	template <class U> static handle<T> cast_static		(const loose_handle<U> &x);
+	template <class U> static handle<T> cast_dynamic	(const loose_handle<U> &x);
+	template <class U> static handle<T> cast_const		(const loose_handle<U> &x);
+	template <class U> static handle<T> cast_reinterpret(const loose_handle<U> &x);
 
-	template<class U> static handle<T> cast_static(const rhandle<U> &x);
-	template<class U> static handle<T> cast_dynamic(const rhandle<U> &x);
-	template<class U> static handle<T> cast_const(const rhandle<U> &x);
-	template<class U> static handle<T> cast_reinterpret(const rhandle<U> &x);
+	template <class U> static handle<T> cast_static		(const rhandle<U> &x);
+	template <class U> static handle<T> cast_dynamic	(const rhandle<U> &x);
+	template <class U> static handle<T> cast_const		(const rhandle<U> &x);
+	template <class U> static handle<T> cast_reinterpret(const rhandle<U> &x);
 
-	template<class U> static handle<T> cast_static(U* x);
-	template<class U> static handle<T> cast_dynamic(U* x);
-	template<class U> static handle<T> cast_const(U* x);
-	template<class U> static handle<T> cast_reinterpret(U* x);
+	template <class U> static handle<T> cast_static		(U* x);
+	template <class U> static handle<T> cast_dynamic	(U* x);
+	template <class U> static handle<T> cast_const		(U* x);
+	template <class U> static handle<T> cast_reinterpret(U* x);
 
 	//! Returns pointer to the object that is being wrapped
 	pointer get()const { return obj; }
@@ -370,8 +380,8 @@ public:
 	using handle<value_type>::unique;
 	using handle<value_type>::operator bool;
 	using handle<value_type>::get;
-	using handle<value_type>::operator *;
-	using handle<value_type>::operator ->;
+	using handle<value_type>::operator*;
+	using handle<value_type>::operator->;
 
 	/*
 	operator const handle<value_type>&()const
@@ -386,7 +396,7 @@ private:
 
 	void add_to_rlist()
 	{
-//		value_type*& obj(handle<T>::obj); // Required to keep gcc 3.4.2 form barfing
+//		value_type*& obj(handle<T>::obj); // Required to keep gcc 3.4.2 from barfing
 
 		assert(obj);
 		obj->rref();
@@ -407,7 +417,7 @@ private:
 
 	void del_from_rlist()
 	{
-//		value_type*& obj(handle<T>::obj); // Required to keep gcc 3.4.2 form barfing
+//		value_type*& obj(handle<T>::obj); // Required to keep gcc 3.4.2 from barfing
 		assert(obj);
 		obj->runref();
 
@@ -438,20 +448,20 @@ public:
 	//! Constructor that constructs from a pointer to new object
 	rhandle(pointer x):handle<T>(x)
 	{
-//		value_type*& obj(handle<T>::obj); // Required to keep gcc 3.4.2 form barfing
+//		value_type*& obj(handle<T>::obj); // Required to keep gcc 3.4.2 from barfing
 		if(obj)add_to_rlist();
 	}
 
 	rhandle(const handle<value_type> &x):handle<T>(x)
 	{
-//		value_type*& obj(handle<T>::obj); // Required to keep gcc 3.4.2 form barfing
+//		value_type*& obj(handle<T>::obj); // Required to keep gcc 3.4.2 from barfing
 		if(obj)add_to_rlist();
 	}
 
 	//! Default copy constructor
 	rhandle(const rhandle<value_type> &x):handle<T>(x)
 	{
-//		value_type*& obj(handle<T>::obj); // Required to keep gcc 3.4.2 form barfing
+//		value_type*& obj(handle<T>::obj); // Required to keep gcc 3.4.2 from barfing
 		if(obj)add_to_rlist();
 	}
 
@@ -485,7 +495,7 @@ public:
 	rhandle<value_type> &
 	operator=(const rhandle<value_type> &x)
 	{
-//		value_type*& obj(handle<T>::obj); // Required to keep gcc 3.4.2 form barfing
+//		value_type*& obj(handle<T>::obj); // Required to keep gcc 3.4.2 from barfing
 		if(x.get()==obj)
 			return *this;
 
@@ -503,7 +513,7 @@ public:
 	rhandle<value_type>&
 	operator=(const handle<value_type> &x)
 	{
-//		value_type*& obj(handle<T>::obj); // Required to keep gcc 3.4.2 form barfing
+//		value_type*& obj(handle<T>::obj); // Required to keep gcc 3.4.2 from barfing
 		if(x.get()==obj)
 			return *this;
 
@@ -521,7 +531,7 @@ public:
 	rhandle<value_type>&
 	operator=(value_type* x)
 	{
-//		value_type*& obj(handle<T>::obj); // Required to keep gcc 3.4.2 form barfing
+//		value_type*& obj(handle<T>::obj); // Required to keep gcc 3.4.2 from barfing
 		if(x==obj)
 			return *this;
 
@@ -541,7 +551,7 @@ public:
 	void
 	detach()
 	{
-//		value_type*& obj(handle<T>::obj); // Required to keep gcc 3.4.2 form barfing
+//		value_type*& obj(handle<T>::obj); // Required to keep gcc 3.4.2 from barfing
 		if(obj)del_from_rlist();
 		handle<value_type>::detach();
 		obj=0;
@@ -560,7 +570,7 @@ public:
 	count_type
 	rcount()const
 	{
-//		value_type*const& obj(handle<T>::obj); // Required to keep gcc 3.4.2 form barfing
+//		value_type*const& obj(handle<T>::obj); // Required to keep gcc 3.4.2 from barfing
 		return obj?obj->rcount():0;
 	}
 
@@ -568,14 +578,14 @@ public:
 	bool
 	runique()const
 	{
-//		value_type*& obj(handle<T>::obj); // Required to keep gcc 3.4.2 form barfing
+//		value_type*& obj(handle<T>::obj); // Required to keep gcc 3.4.2 from barfing
 		assert(obj); return obj->front_==obj->back_;
 	}
 
 	//! \writeme
 	int replace(const handle<value_type> &x)
 	{
-//		value_type*& obj(handle<T>::obj); // Required to keep gcc 3.4.2 form barfing
+//		value_type*& obj(handle<T>::obj); // Required to keep gcc 3.4.2 from barfing
 		assert(obj);
 		assert(x.get()!=obj);
 
@@ -615,7 +625,7 @@ public:
 	{
 		assert(0);
 		pointer ptr=x.obj;
-		x.obj=x.get();
+		x.obj=obj;
 		obj=ptr;
 		return *this;
 	}
@@ -623,11 +633,6 @@ public:
 }; // END of template class rhandle
 
 
-
-
-
-
-
 // ========================================================================
 /*!	\class	loose_handle _handle.h	ETL/handle
 **	\brief	Loose Object Handle
@@ -648,6 +653,9 @@ public:
 	typedef int size_type;
 
 protected:
+#ifdef _DEBUG
+public:
+#endif
 	value_type *obj;		//!< Pointer to object
 
 public:
@@ -699,7 +707,7 @@ public:
 	swap(loose_handle<value_type> &x)
 	{
 		pointer ptr=x.obj;
-		x.obj=x.get();
+		x.obj=obj;
 		obj=ptr;
 		return *this;
 	}
@@ -763,141 +771,53 @@ public:
 	bool unref() { if(obj && !obj->unref()){ obj=0; return false; } return true; }
 }; // END of template class loose_handle
 
+// cast loose_handle<> -> handle<>
+template <class T> template <class U> handle<T> handle<T>::cast_static	   (const loose_handle<U>& x) { return handle<T>(static_cast	 <T*>(x.get())); }
+template <class T> template <class U> handle<T> handle<T>::cast_dynamic	   (const loose_handle<U>& x) { return handle<T>(dynamic_cast	 <T*>(x.get())); }
+template <class T> template <class U> handle<T> handle<T>::cast_const	   (const loose_handle<U>& x) { return handle<T>(const_cast		 <T*>(x.get())); }
+template <class T> template <class U> handle<T> handle<T>::cast_reinterpret(const loose_handle<U>& x) { return handle<T>(reinterpret_cast<T*>(x.get())); }
 
+// cast rhandle_handle<> -> handle<>
+template <class T> template <class U> handle<T> handle<T>::cast_static	   (const rhandle<U>&	   x) { return handle<T>(static_cast	 <T*>(x.get())); }
+template <class T> template <class U> handle<T> handle<T>::cast_dynamic	   (const rhandle<U>&	   x) { return handle<T>(dynamic_cast	 <T*>(x.get())); }
+template <class T> template <class U> handle<T> handle<T>::cast_const	   (const rhandle<U>&	   x) { return handle<T>(const_cast		 <T*>(x.get())); }
+template <class T> template <class U> handle<T> handle<T>::cast_reinterpret(const rhandle<U>&	   x) { return handle<T>(reinterpret_cast<T*>(x.get())); }
 
+// cast U* -> handle<>
+template <class T> template <class U> handle<T> handle<T>::cast_static	   (U*					   x) { return handle<T>(static_cast	 <T*>(x));		 }
+template <class T> template <class U> handle<T> handle<T>::cast_dynamic	   (U*					   x) { return handle<T>(dynamic_cast	 <T*>(x));		 }
+template <class T> template <class U> handle<T> handle<T>::cast_const	   (U*					   x) { return handle<T>(const_cast		 <T*>(x));		 }
+template <class T> template <class U> handle<T> handle<T>::cast_reinterpret(U*					   x) { return handle<T>(reinterpret_cast<T*>(x));		 }
 
-template<class T> template<class U> handle<T>
-handle<T>::cast_static(const loose_handle<U> &x)
-	{ return handle<T>(static_cast<T*>(x.get())); }
+// operator== for handle<>, loose_handle<> and T*
+template <class T,class U> bool operator==(const handle		 <T>& lhs,const handle		<U>& rhs) { return (lhs.get()==rhs.get()); }
+template <class T,class U> bool operator==(const loose_handle<T>& lhs,const loose_handle<U>& rhs) { return (lhs.get()==rhs.get()); }
+template <class T,class U> bool operator==(const handle		 <T>& lhs,const loose_handle<U>& rhs) { return (lhs.get()==rhs.get()); }
+template <class T,class U> bool operator==(const loose_handle<T>& lhs,const handle		<U>& rhs) { return (lhs.get()==rhs.get()); }
+template <class T>		   bool operator==(const handle<T>&		  lhs,const T*				 rhs) { return (lhs.get()==rhs);	   }
+template <class T>		   bool operator==(const loose_handle<T>& lhs,const T*				 rhs) { return (lhs.get()==rhs);	   }
+template <class T>		   bool operator==(const T*				  lhs,const handle<T>&		 rhs) { return (lhs		 ==rhs.get()); }
+template <class T>		   bool operator==(const T*				  lhs,const loose_handle<T>& rhs) { return (lhs		 ==rhs.get()); }
 
-template <class T> template <class U> handle<T>
-handle<T>::cast_dynamic(const loose_handle<U> &x)
-	{ return handle<T>(dynamic_cast<T*>(x.get())); }
+// operator!= for handle<>, loose_handle<> and T*
+template <class T,class U> bool operator!=(const handle		 <T>& lhs,const handle		<U>& rhs) { return (lhs.get()!=rhs.get()); }
+template <class T,class U> bool operator!=(const loose_handle<T>& lhs,const loose_handle<U>& rhs) { return (lhs.get()!=rhs.get()); }
+template <class T,class U> bool operator!=(const handle		 <T>& lhs,const loose_handle<U>& rhs) { return (lhs.get()!=rhs.get()); }
+template <class T,class U> bool operator!=(const loose_handle<T>& lhs,const handle		<U>& rhs) { return (lhs.get()!=rhs.get()); }
+template <class T>		   bool operator!=(const handle<T>&		  lhs,const T*				 rhs) { return (lhs.get()!=rhs);	   }
+template <class T>		   bool operator!=(const loose_handle<T>& lhs,const T*				 rhs) { return (lhs.get()!=rhs);	   }
+template <class T>		   bool operator!=(const T*				  lhs,const handle<T>&		 rhs) { return (lhs		 !=rhs.get()); }
+template <class T>		   bool operator!=(const T*				  lhs,const loose_handle<T>& rhs) { return (lhs		 !=rhs.get()); }
 
-template <class T> template <class U> handle<T>
-handle<T>::cast_const(const loose_handle<U> &x)
-	{ return handle<T>(const_cast<T*>(x.get())); }
-
-template <class T> template <class U> handle<T>
-handle<T>::cast_reinterpret(const loose_handle<U> &x)
-	{ return handle<T>(reinterpret_cast<T*>(x.get())); }
-
-
-
-template <class T> template <class U> handle<T>
-handle<T>::cast_static(const rhandle<U> &x)
-	{ return handle<T>(static_cast<T*>(x.get())); }
-
-template <class T> template <class U> handle<T>
-handle<T>::cast_dynamic(const rhandle<U> &x)
-	{ return handle<T>(dynamic_cast<T*>(x.get())); }
-
-template <class T> template <class U> handle<T>
-handle<T>::cast_const(const rhandle<U> &x)
-	{ return handle<T>(const_cast<T*>(x.get())); }
-
-template <class T> template <class U> handle<T>
-handle<T>::cast_reinterpret(const rhandle<U> &x)
-	{ return handle<T>(reinterpret_cast<T*>(x.get())); }
-
-
-
-template <class T> template <class U> handle<T>
-handle<T>::cast_static(U* x)
-	{ return handle<T>(static_cast<T*>(x)); }
-
-template <class T> template <class U> handle<T>
-handle<T>::cast_dynamic(U* x)
-	{ return handle<T>(dynamic_cast<T*>(x)); }
-
-template <class T> template <class U> handle<T>
-handle<T>::cast_const(U* x)
-	{ return handle<T>(const_cast<T*>(x)); }
-
-template <class T> template <class U> handle<T>
-handle<T>::cast_reinterpret(U* x)
-	{ return handle<T>(reinterpret_cast<T*>(x)); }
-
-
-
-
-
-template <class T,class U> bool
-operator==(const handle<T> &lhs,const handle<U> &rhs)
-	{ return (lhs.get()==rhs.get()); }
-template <class T,class U> bool
-operator==(const loose_handle<T> &lhs,const loose_handle<U> &rhs)
-	{ return (lhs.get()==rhs.get()); }
-template <class T,class U> bool
-operator==(const handle<T> &lhs,const loose_handle<U> &rhs)
-	{ return (lhs.get()==rhs.get()); }
-template <class T,class U> bool
-operator==(const loose_handle<T> &lhs,const handle<U> &rhs)
-	{ return (lhs.get()==rhs.get()); }
-template <class T> bool
-operator==(const handle<T> &lhs,const T *rhs)
-	{ return (lhs.get()==rhs); }
-template <class T> bool
-operator==(const loose_handle<T> &lhs,const T *rhs)
-	{ return (lhs.get()==rhs); }
-template <class T> bool
-operator==(const T *lhs,const handle<T> &rhs)
-	{ return (lhs==rhs.get()); }
-template <class T> bool
-operator==(const T *lhs,const loose_handle<T> &rhs)
-	{ return (lhs==rhs.get()); }
-
-
-template <class T,class U> bool
-operator!=(const handle<T> &lhs,const handle<U> &rhs)
-	{ return (lhs.get()!=rhs.get()); }
-template <class T,class U> bool
-operator!=(const loose_handle<T> &lhs,const loose_handle<U> &rhs)
-	{ return (lhs.get()!=rhs.get()); }
-template <class T,class U> bool
-operator!=(const handle<T> &lhs,const loose_handle<U> &rhs)
-	{ return (lhs.get()!=rhs.get()); }
-template <class T,class U> bool
-operator!=(const loose_handle<T> &lhs,const handle<U> &rhs)
-	{ return (lhs.get()!=rhs.get()); }
-template <class T> bool
-operator!=(const handle<T> &lhs,const T *rhs)
-	{ return (lhs.get()!=rhs); }
-template <class T> bool
-operator!=(const loose_handle<T> &lhs,const T *rhs)
-	{ return (lhs.get()!=rhs); }
-template <class T> bool
-operator!=(const T *lhs,const handle<T> &rhs)
-	{ return (lhs!=rhs.get()); }
-template <class T> bool
-operator!=(const T *lhs,const loose_handle<T> &rhs)
-	{ return (lhs!=rhs.get()); }
-
-
-template <class T,class U> bool
-operator<(const handle<T> &lhs,const handle<U> &rhs)
-	{ return (lhs.get()<rhs.get()); }
-template <class T,class U> bool
-operator<(const loose_handle<T> &lhs,const loose_handle<U> &rhs)
-	{ return (lhs.get()<rhs.get()); }
-template <class T,class U> bool
-operator<(const handle<T> &lhs,const loose_handle<U> &rhs)
-	{ return (lhs.get()<rhs.get()); }
-template <class T,class U> bool
-operator<(const loose_handle<T> &lhs,const handle<U> &rhs)
-	{ return (lhs.get()<rhs.get()); }
-template <class T> bool
-operator<(const handle<T> &lhs,const T *rhs)
-	{ return (lhs.get()<rhs); }
-template <class T> bool
-operator<(const loose_handle<T> &lhs,const T *rhs)
-	{ return (lhs.get()<rhs); }
-template <class T> bool
-operator<(const T *lhs,const handle<T> &rhs)
-	{ return (lhs<rhs.get()); }
-template <class T> bool
-operator<(const T *lhs,const loose_handle<T> &rhs)
-	{ return (lhs<rhs.get()); }
+// operator< for handle<>, loose_handle<> and T*
+template <class T,class U> bool operator<(const handle<T>&		  lhs,const handle<U>&		 rhs) { return (lhs.get()<rhs.get());  }
+template <class T,class U> bool operator<(const loose_handle<T>&  lhs,const loose_handle<U>& rhs) { return (lhs.get()<rhs.get());  }
+template <class T,class U> bool operator<(const handle<T>&		  lhs,const loose_handle<U>& rhs) { return (lhs.get()<rhs.get());  }
+template <class T,class U> bool operator<(const loose_handle<T>&  lhs,const handle<U>&		 rhs) { return (lhs.get()<rhs.get());  }
+template <class T>		   bool operator<(const handle<T>&		  lhs,const T*				 rhs) { return (lhs.get()<rhs);		   }
+template <class T>		   bool operator<(const loose_handle<T>&  lhs,const T*				 rhs) { return (lhs.get()<rhs);		   }
+template <class T>		   bool operator<(const T*				  lhs,const handle<T>&		 rhs) { return (lhs		 <rhs.get());  }
+template <class T>		   bool operator<(const T*				  lhs,const loose_handle<T>& rhs) { return (lhs		 <rhs.get());  }
 
 _ETL_END_NAMESPACE
 
