@@ -1,9 +1,10 @@
 /*! ========================================================================
 ** Extended Template and Library
 ** State Machine Abstraction Class Implementation
-** $Id: _smach.h 659 2007-09-12 21:01:33Z dooglus $
+** $Id$
 **
 ** Copyright (c) 2002 Robert B. Quattlebaum Jr.
+** Copyright (c) 2008 Chris Moore
 **
 ** This package is free software; you can redistribute it and/or
 ** modify it under the terms of the GNU General Public License as
@@ -98,7 +99,7 @@ public:
 
 	//! Event definition class
 	template<typename T>
-	class event_def
+	class event_def_internal
 	{
 		// List our friends
 		friend class smach;
@@ -118,11 +119,11 @@ public:
 	public:
 
 		//! Less-than operator for sorting. Based on event_key value.
-		bool operator<(const event_def &rhs)const
+		bool operator<(const event_def_internal &rhs)const
 			{ return id<rhs.id; }
 
 		//! Equal-to operator. Based on event_key value.
-		bool operator==(const event_def &rhs)const
+		bool operator==(const event_def_internal &rhs)const
 			{ return id==rhs.id; }
 
 		//! Less-than operator for finding.
@@ -134,13 +135,13 @@ public:
 			{ return id==rhs; }
 
 		//! Trivial Constructor
-		event_def() { }
+		event_def_internal() { }
 
-		//! Constructor for creating an event_def from the given key and function reference.
-		event_def(event_key a, funcptr b):id(a),handler(b) { }
+		//! Constructor for creating an event_def_internal from the given key and function reference.
+		event_def_internal(event_key a, funcptr b):id(a),handler(b) { }
 
 		//! Copy constructor
-		event_def(const event_def &x):id(x.id),handler(x.handler) { }
+		event_def_internal(const event_def_internal &x):id(x.id),handler(x.handler) { }
 
 	};
 
@@ -168,7 +169,7 @@ public:
 		friend class smach;
 
 	public:
-		typedef event_def<T> event_def;
+		typedef event_def_internal<T> event_def;
 		typedef T state_context_type;
 
 
