@@ -1,7 +1,7 @@
 /* ========================================================================
 ** Extended Template and Library
 ** Abstraction for a Generic Value Type
-** $Id: _value.h 334 2007-03-16 00:37:48Z dooglus $
+** $Id$
 **
 ** Copyright (c) 2002 Adrian Bentley
 **
@@ -24,8 +24,8 @@
 
 /* === S T A R T =========================================================== */
 
-#ifndef __ETL_VALUE_H
-#define __ETL_VALUE_H
+#ifndef __ETL__VALUE_H
+#define __ETL__VALUE_H
 
 /* === H E A D E R S ======================================================= */
 #include <algorithm>
@@ -73,7 +73,7 @@ class value
 
 	contentholder	*content;
 
-public:	//structor interface
+public:	//constructor interface
 	value()
 		:content(0)
 	{
@@ -134,7 +134,7 @@ private: //implementation interface
 	public: //representation
 		T	obj;
 
-	public: //structor interface
+	public: //constructor interface
 
 		holder(const T &o)
 			:obj(o)
